@@ -162,8 +162,8 @@ std::vector<std::vector<int>> FastFeatureBundling(std::vector<std::unique_ptr<Bi
     sorted_idx.emplace_back(i);
   }
   // sort by non zero cnt, bigger first
-  std::sort(sorted_idx.begin(), sorted_idx.end(),
-            [&feature_non_zero_cnt](int a, int b) {
+  std::stable_sort(sorted_idx.begin(), sorted_idx.end(),
+                   [&feature_non_zero_cnt](int a, int b) {
     return feature_non_zero_cnt[a] > feature_non_zero_cnt[b];
   });
 
@@ -226,14 +226,11 @@ void Dataset::Construct(
     }
   }
   if (used_features.empty()) {
-    Log::Fatal("Cannot construct Dataset since there are no useful features.\n"
-               "It should be at least two unique rows.\n"
-               "If the num_row (num_data) is small, you can set min_data=1 and min_data_in_bin=1 to fix this.\n"
-               "Otherwise, please make sure you are using the right dataset");
+    Log::Warning("There are no meaningful features, as all feature values are constant.");
   }
   auto features_in_group = NoGroup(used_features);
 
-  if (io_config.enable_bundle) {
+  if (io_config.enable_bundle && !used_features.empty()) {
     features_in_group = FastFeatureBundling(bin_mappers,
                                             sample_non_zero_indices, num_per_col, total_sample_cnt,
                                             used_features, io_config.max_conflict_rate,
@@ -323,14 +320,16 @@ void Dataset::Construct(
 
 void Dataset::FinishLoad() {
   if (is_finish_load_) { return; }
-  OMP_INIT_EX();
-  #pragma omp parallel for schedule(guided)
-  for (int i = 0; i < num_groups_; ++i) {
-    OMP_LOOP_EX_BEGIN();
-    feature_groups_[i]->bin_data_->FinishLoad();
-    OMP_LOOP_EX_END();
+  if (num_groups_ > 0) {
+    OMP_INIT_EX();
+#pragma omp parallel for schedule(guided)
+    for (int i = 0; i < num_groups_; ++i) {
+      OMP_LOOP_EX_BEGIN();
+      feature_groups_[i]->bin_data_->FinishLoad();
+      OMP_LOOP_EX_END();
+    }
+    OMP_THROW_EX();
   }
-  OMP_THROW_EX();
   is_finish_load_ = true;
 }
 
