@@ -79,8 +79,8 @@ public:
     for (data_size_t i = 0; i < num_data; ++i) {
       sorted_idx.emplace_back(i);
     }
-    std::sort(sorted_idx.begin(), sorted_idx.end(),
-              [score](data_size_t a, data_size_t b) {return score[a] > score[b]; });
+    std::stable_sort(sorted_idx.begin(), sorted_idx.end(),
+                     [score](data_size_t a, data_size_t b) {return score[a] > score[b]; });
 
     int num_hit = 0;
     double sum_ap = 0.0f;
